if (not Numen) then return; end

local Numen = Numen;

local AceOO = AceLibrary("AceOO-2.0")

Numen.Button = AceOO.Class("AceEvent-2.0");

local _G = getfenv(0)

local function SwapTexture(button, name, value)
	if (name=="state-parent") then
		if (value ~= "0") then 
			if (button.popButtons[tonumber(value)]) then
				popButton = button.popButtons[tonumber(value)].button;
				button.texture:SetTexture(popButton.texture:GetTexture());
				button.spellID = popButton.spellID;
				button.class.spellID = popButton.spellID;
				button.class:UpdateCooldown();
				Numen:UpdateMacro();
			end
			Numen.db.profile.totemStates[tostring(button.index)] = value;
		end
	end
end

function Numen.Button.prototype:init(idx, type, order)
	Numen.Button.super.prototype.init(self);
	
	self.index = idx;
	self.order = order;
	self.frame = CreateFrame("Frame", "Numen"..type.."Frame", Numen.root, "SecureStateHeaderTemplate");
	self.button = CreateFrame("CheckButton", "Numen"..type.."Button", self.frame, "SecureAnchorEnterTemplate,SecureActionButtonTemplate,ActionButtonTemplate");
	self.button.class = self;
	self.button:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 0, 0);
	self.button:SetWidth(36);
	self.button:SetHeight(36);
	self.button:SetAttribute("type", "spell");
	self.button:SetAttribute("childstate-onenter", "show"..type);
	self.button:SetAttribute("childstate-onleave", "hide"..type);
	self.button:SetAttribute("anchorchild", Numen.root);
	self.button:SetAttribute("totemType", type);
	self.button:HookScript("OnEnter", function(b) Numen:ShowTooltip(b); end);
	self.button:HookScript("OnLeave", function() Numen:HideTooltip(); end);
	self.button.texture = self.button:CreateTexture(nil, "BACKGROUND");
	self.button:SetCheckedTexture("");
	self.button:SetNormalTexture("");
	self:ApplyStyle();
	self.button.texture:SetPoint("TOPLEFT", self.button, "TOPLEFT", 3, -3);
	self.button.texture:SetPoint("BOTTOMRIGHT", self.button, "BOTTOMRIGHT",  -3, 3);
	self.button.texture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	self.frame:SetAttribute("addchild", self.button);
	Numen.root:SetAttribute("statemap-anchor-show"..type, "*:"..idx);
	Numen.root:SetAttribute("statemap-anchor-hide"..type, ";");
	Numen.root:SetAttribute("delaystatemap-anchor-hide"..type, idx..":0");
	Numen.root:SetAttribute("delaytimemap-anchor-hide"..type,  idx..":0.5");
	Numen.root:SetAttribute("delayhovermap-anchor-hide"..type, idx..":true");
	self.statestr = "";

	-- Initialize Popups
	self.popButtons = {};
	for spellName, args in pairs(Numen.totems[type].spells) do
		self.popButtons[args.gIndex] = Numen.PopButton:new(self, type, spellName, args);
	end
	self.button.popButtons = self.popButtons;
	self.button.index = self.index;
	self.button:SetScript("OnAttributeChanged", SwapTexture);
	self.frame:SetAttribute("state", Numen.db.profile.totemStates[tostring(idx)] or "1");
	
	self.button:SetAttribute("statebutton", self.statestr);
	
	self.button.cooldown = _G[self.button:GetName().."Cooldown"];
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN", "ButtonEventHandler")
end

function Numen.Button.prototype:UpdateLayout(x, y, isVert, isRtDn)
	self.frame:ClearAllPoints();
	self.frame:SetPoint("TOPLEFT", Numen.root, "TOPLEFT", x, y);
	local scale = Numen.db.profile.confScale;
	for idx, popButtonClass in pairs(self.popButtons) do
		local x, y = 0, 0;
		local order = popButtonClass.button:GetAttribute("order");
		local padding = Numen.db.profile.confPadding;
		if (isVert) then
			x = (36+padding)*order - 36;
			if (not isRtDn) then
				x = x*(-1);
			end
		else
			y = (-36-padding)*order + 36;
			if (not isRtDn) then
				y = y*(-1);
			end
		end
		popButtonClass:UpdateLayout(isVert, isRtDn, x/scale, y/scale);
	end
	self.button:SetAlpha(Numen.db.profile.confAlpha);
end

function Numen.Button.prototype:ApplyStyle()
	local frame = CreateFrame("Frame", self.button:GetName().."DL", self.button);
	frame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 1, edgeFile = "", edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0},})
	frame:SetBackdropColor(0, 0, 0, 0.6)
	frame:SetAllPoints(self.button)
	frame:SetFrameLevel(0)
end

function Numen.Button.prototype:ShowPops()
	for idx, popButtonClass in pairs (self.popButtons) do
		popButtonClass.button:Show();
--[[		totemPop.handle:EnableMouse(true);
		totemPop.handle:SetScript("OnClick", function(f, b, d) Numen:OnClickPopButton(f, b, d); end);
		totemPop.handle.texture:SetTexture(0, 1, 0, 0.25);
		totemPop.handle:SetFrameLevel(4);
--]]
	end
end

function Numen.Button.prototype:UpdateCooldown()
	if (self.button.spellID and self.button.cooldown) then
		local start, duration, enable = GetSpellCooldown(self.button.spellID, BOOKTYPE_SPELL)
		CooldownFrame_SetTimer(self.button.cooldown, start, duration, enable)
	end
end

function Numen.Button.prototype:ButtonEventHandler(e)
	if (event == "ACTIONBAR_UPDATE_COOLDOWN") then
		self:UpdateCooldown();
	end
end

